#    Society Self-Service
#    Copyright © 2018-2019  Yingtong Li (RunasSudo)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.

from googleapiclient.discovery import build

from django.contrib.auth.decorators import login_required

from django.conf import settings
from django.http import HttpResponse
from django.shortcuts import render, redirect
from django.urls import reverse
from django.utils import timezone

from . import models

import datetime
import json

@login_required
def index(request):
	return render(request, 'sspromotions/index.html')

def get_calendar_events(calbegin, calend):
	service = build('calendar', 'v3', developerKey=settings.GOOGLE_API_KEY)
	events = service.events().list(
		calendarId=settings.GOOGLE_CALENDAR_ID,
		orderBy='startTime',
		singleEvents=True,
		timeMin=datetime.datetime.combine(calbegin, datetime.time(tzinfo=timezone.get_current_timezone())).isoformat(),
		timeMax=datetime.datetime.combine(calend, datetime.time(tzinfo=timezone.get_current_timezone())).isoformat()
	).execute()
	for event in events['items']:
		yield {
			'date': datetime.datetime.strptime(event['start']['date'], '%Y-%m-%d') if 'date' in event['start'] else datetime.datetime.strptime(event['start']['dateTime'][:-3] + event['start']['dateTime'][-2:], '%Y-%m-%dT%H:%M:%S%z'),
			'title': event['summary'],
			'link': event['htmlLink']
		}

@login_required
def bulletin_list(request):
	items_past = []
	items_upcoming = []
	items_future = []
	
	dtbegin = timezone.now().date()
	dtend = dtbegin + datetime.timedelta(days=7)
	
	for item in models.BulletinItem.objects.all():
		if not item.can_user_access(request.user):
			continue
		
		if item.date >= dtbegin and item.date < dtend:
			items_upcoming.append(item)
		elif item.date >= dtend:
			items_future.append(item)
		else:
			items_past.append(item)
	
	return render(request, 'sspromotions/bulletin_list.html', {
		'items_past': items_past,
		'items_upcoming': items_upcoming,
		'items_future': items_future
	})

@login_required
def bulletin_preview(request):
	calbegin = timezone.now().date() + datetime.timedelta(days=1) # Start tomorrow for calendar
	calend = calbegin + datetime.timedelta(days=14)
	
	bulbegin = timezone.now().date()
	bulend = bulbegin + datetime.timedelta(days=7)
	
	return render(request, 'sspromotions/email/bulletin.html', {
		'events': get_calendar_events(calbegin, calend),
		'groups': models.Group.objects.all(),
		'more': [],
		'bulbegin': bulbegin,
		'bulend': bulend
	})

@login_required
def bulletin_new(request):
	if request.method == 'POST':
		item = models.BulletinItem()
		item.author = request.user
		item.group = models.Group.objects.get(id=int(request.POST['group']))
		item.title = request.POST['title']
		item.date = request.POST['date']
		item.content = request.POST['content']
		item.link = request.POST['link']
		if request.FILES:
			item.image = request.FILES['image']
		item.also_limit = [int(k[11:]) for k, v in request.POST.items() if k.startswith('also_limit_') and v]
		item.save()
		
		if request.POST['submit'] == 'Save':
			return redirect(reverse('bulletin_list'))
		else:
			return redirect(reverse('bulletin_edit', kwargs={'id': item.id}))
	else:
		item = models.BulletinItem()
		item.author = request.user
		item.date = timezone.now().date()
		item.date += datetime.timedelta(days=(6 - item.date.weekday() + 7) % 7) # Next Sunday (6 = Sunday)
		return render(request, 'sspromotions/bulletin_edit.html', {
			'item': item,
			'groups': models.Group.objects.all()
		})

@login_required
def bulletin_edit(request, id):
	if request.method == 'POST':
		item = models.BulletinItem.objects.get(id=id)
		item.group = models.Group.objects.get(id=int(request.POST['group']))
		if not item.can_user_access(request.user):
			return HttpResponse('Unauthorized', status=401)
		item.title = request.POST['title']
		item.date = request.POST['date']
		item.content = request.POST['content']
		item.link = request.POST['link']
		if request.FILES:
			item.image = request.FILES['image']
		item.also_limit = [int(k[11:]) for k, v in request.POST.items() if k.startswith('also_limit_') and v]
		item.save()
		
		if request.POST['submit'] == 'Save':
			return redirect(reverse('bulletin_list'))
		else:
			return redirect(reverse('bulletin_edit', kwargs={'id': item.id}))
	else:
		item = models.BulletinItem.objects.get(id=id)
		if not item.can_user_access(request.user):
			return HttpResponse('Unauthorized', status=401)
		return render(request, 'sspromotions/bulletin_edit.html', {
			'item': item,
			'groups': models.Group.objects.all()
		})

@login_required
def bulletin_delete(request, id):
	item = models.BulletinItem.objects.get(id=id)
	if not item.can_user_access(request.user):
		return HttpResponse('Unauthorized', status=401)
	item.delete()
	
	return redirect(reverse('bulletin_list'))
